function global_score=alc(cash_history, auc_history, initial_budget, axis_scale)
%global_score=alc(cash_history, auc_history, initial_budget, axis_scale)
% Compute the Area under the Learning Curve (ALC)
% Inputs:
% cash_history --       History of virtual cash spent (number of labels queried)
% auc_history --        Corresponding AUC values
% initial_budget --     Initial budget (total number of training examples)
% axis_scale --         Scale of the x axis 'linear, 'sqrt, or 'log'
% Returns:
% global_score --       Normalized ALC

% Author: Isabelle Guyon -- November 2009 -- causality@clopinet.com

    % Last point extrapolation
    last_point=auc_history(length(auc_history));
    x=scale_axis(axis_scale, [cash_history(1:length(cash_history)-1) initial_budget]);
    y=[auc_history last_point];
    % Compute the score
    s=0;
    % Integrate by the trapeze method
    for k=2:length(x)
        deltax=x(k)-x(k-1);
        mu=(y(k)+y(k-1))/2;
        s=s+deltax*mu;
    end
    % Normalize the score
    bias=scale_axis(axis_scale, initial_budget)*0.5;
    norma=bias;

    global_score=(s-bias)/norma;
    
return
    