function [label_num, chunk_num]=choose_data_split(ml_algo, axis_scale, P)
%[label_num, chunk_num]=choose_data_split(ml_algo, axis_scale, P)
% Input:
% ml_algo       -- Name of a learning algorithm (e.g. a CLOP object)
% axis_scale    -- 'linear', 'sqrt', or 'log' scaling of the x axis
% P             -- Total number of training examples
% Returns:
% label_num     -- A vector containing the number of labels at each iteration
% chunk_num     -- The length of label_num

% Author: Isabelle Guyon -- November 2009 -- causality@clopinet.com

    if strcmp(ml_algo, 'zarbip')
        chunk_num=1;
        label_num=P-1;
        return
    end
    if strcmp(axis_scale, 'log') % exponential chunks
        chunk_num=round(log2(P));
        label_num=2.^(0:chunk_num-1);
        label_num(chunk_num)=label_num(chunk_num)+P-2^chunk_num;
    elseif strcmp(axis_scale, 'sqrt') % square chunks
        chunk_num=10;
        ln=(1:chunk_num+1).^2;
        lnn=sum(ln);
        ln=floor(ln*P/lnn);
        lnn=sum(ln)-ln(chunk_num);
        ln(chunk_num)=P-lnn+ln(1)-1;
        label_num(1:chunk_num)=ln(2:chunk_num+1);
    else
        chunk_num=10;
        label_num(1:chunk_num-1)=floor(P/chunk_num);
        label_num(chunk_num)=P-(chunk_num-1)*label_num(1)-1;
    end
 return