function [Y, S, score, global_score, total_cost, stop]=get_answer(answer_dir, query_name, data_name, Predicted_values, Samples_queried, cash_history, auc_history, initial_budget, axis_scale, known_idx) 
%[Y, S, score, global_score, total_cost, stop]=get_answer(answer_dir, query_name, data_name, Predicted_values, Samples_queried, cash_history, auc_history, initial_budget, axis_scale, known_idx) 
% Get the labels Y resulting from the query of samples S
% and the current AUC score and the global score
% The function was initially written with the 2 arguments:
% answer_dir -- directory where the answers are stored
% query_name -- name of the query
% Returns:
% Y --          Labels queried
% S --          Corresponding sample indices (raws of X)
% score --      AUC score
% global_score -- Normalized ALC
% total_cost -- Cost of the query
% stop --       Command to stop if we are in interactive mode with the user
%
% We added for the back door calculations if GLOP does not work:
% data_name, Predicted_values, Samples_queried, cash_history, auc_history,
% initial_budget, axis_scale, known_idx
% This allows computing the results directly without processing
% the query with GLOP. This is dangerous however because then there
% is no error checking.

% Author: Isabelle Guyon -- November 2009 -- causality@clopinet.com

    Y=[]; S=[]; score=0; global_score=0; total_cost=0; stop=0;
    if isempty(query_name)
        % Back door for people who do not have a too old Matlab version to run GLOP 
        % and still want to work on their local computer
        if exist([data_name '.label'], 'file')
            true_labels=load([data_name '.label']);
            unknown_idx=setdiff(1:length(true_labels), known_idx);
            score=auc(Predicted_values(unknown_idx), true_labels(unknown_idx));
            S=Samples_queried;
            Y=true_labels(S);
            total_cost=length(S); % Warning: no check!
            global_score=alc([cash_history total_cost], [auc_history score], initial_budget, axis_scale);
            return
        else
            fprintf('==> Please send submit your query to the server\n    and place your answer in the "Answers" directory.\n');
            query_name = get_query_name(answer_dir);
        end
    end
    if isempty(query_name)
        stop=1;
        return
    end            
    
    answer_name=[answer_dir '/' query_name '.label'];
    sample_name=[answer_dir '/' query_name '.sample'];
    score_name=[answer_dir '/' query_name '.score'];
    global_score_name=[answer_dir '/' query_name '.global_score'];
    total_cost_name=[answer_dir '/' query_name '.totalcost'];
    is_overbudget=[answer_dir '/' query_name '.is_overbudget'];
    if ~exist(answer_name, 'file'), 
        un_zip(answer_dir, query_name);
    end
    if exist(answer_name, 'file')
        Y=load(answer_name); 
        S=load(sample_name); 
    end
    if exist(score_name, 'file')
        score=load(score_name);
        global_score=load(global_score_name);
        total_cost=load(total_cost_name);
    end
    if exist(is_overbudget, 'file')
        warning('==> WARNING: you went over budget...\n');
    end
    re_zip(answer_dir, query_name, 1);
return

function un_zip(file_dir, file_name, rmzip)
%un_zip(file_dir, file_name, rmzip)
% Unzip file file_dir/file_name.zip in file_dir
% remove the zip files if rmzip=1
    if nargin<3, rmzip=0; end
    zipfilename=[file_dir '/' file_name '.zip'];
    unzip(zipfilename, file_dir);
    if rmzip, delete(zipfilename); end
return

function fn=re_zip(file_dir, file_name, rmfiles)
%fn=re_zip(file_dir, file_name, rmfiles)
% Zip files having names file_name.ext found in file_dir
% and remove them if rmfiles=1.
% Return the zip file name.
    if ~usejava('jvm'),
        fprintf('==> Java is not loaded, failed to generate ZIP files !!!\n'); 
        fn=[];
        return
    end
    if nargin<2, rmfiles=0; end
    D=dir([file_dir '/' file_name '.*']);
    [resu_files{1:length(D)}] = deal(D.name);
    resu_files=setdiff(resu_files, [file_name '.zip']);
    zip([file_dir '/' file_name], resu_files, file_dir);
    if rmfiles
        % Remove the original files
        for k=1:length(resu_files)
            delete([file_dir '/' resu_files{k}]);
        end    
    end
    fn=[file_dir '/' file_name '.zip'];
 return
 