function make_query(query_dir, query_name, Yhat, Usample)
%make_query(query_dir, query_name, Yhat, Usample)
% Create a valid active learning challenge query and places the zip
% file in query_dir.
% query_name --     Name of the file
% Yhat --           Prediction values
% Usample --        Samples queried

% Author: Isabelle Guyon -- November 2009 -- causality@clopinet.com

    fprintf('==> Creating a query... ');
    save_vector([query_dir '/' query_name '.predict'], Yhat);
    if ~isempty(Usample)
        save_vector([query_dir '/' query_name '.sample'], Usample);
    end
    rmzip=~exist('whoisglop', 'file');
    fn=re_zip(query_dir, query_name, rmzip);
    if ~isempty(fn)
        fprintf('done\n==> Your query is in %s\n', fn);
    end
return

function fn=re_zip(file_dir, file_name, rmfiles)
%fn=re_zip(file_dir, file_name, rmfiles)
% Zip files having names file_name.ext found in file_dir
% and remove them if rmfiles=1.
% Return the zip file name.
    if ~usejava('jvm'),
        fprintf('==> Java is not loaded, failed to generate ZIP files !!!\n'); 
        fn=[];
        return
    end
    if nargin<2, rmfiles=0; end
    D=dir([file_dir '/' file_name '.*']);
    [resu_files{1:length(D)}] = deal(D.name);
    resu_files=setdiff(resu_files, [file_name '.zip']);
    zip([file_dir '/' file_name], resu_files, file_dir);
    if rmfiles
        % Remove the original files
        for k=1:length(resu_files)
            delete([file_dir '/' resu_files{k}]);
        end    
    end
    fn=[file_dir '/' file_name '.zip'];
 return
 