function h=plot_learning_curve(name, cash_history, auc_history, initial_budget, final_score, axis_scale, nte, old_h)
%h=plot_learning_curve(name, cash_history, auc_history, initial_budget, final_score, axis_scale, nte, old_h)
% Plot the learning curve in the scale specified by axis_scale ('linear', 'sqrt' or 'log')
% Inputs:
% name --           Name of the query
% cash_history --   History of cash spent (a vector)
% auc_history --    Corresponding AUC values
% initial_budget -- Initial budget (number of training examples)
% final_score --    Global score (ALC normalized)
% axis_scale --     'linear', 'sqrt' or 'log'
% nte --            Number of test examples
% old_h --          Handle of a previous plot
% Returns:
% h --      The plot handle

% Author: Isabelle Guyon -- November 2009 -- causality@clopinet.com

    if nargin<5, axis_scale='linear'; end
    if nargin<6, nte=[]; end
    if nargin<7, old_h=[]; end
    % The last point corresponds to perfect predictions if we would buy
    % all the labels
    if ~nte
        last_point=1;
    else
        last_point=auc_history(length(auc_history));
    end
    cash_history(length(cash_history))=initial_budget; 
    auc_history=[auc_history, last_point];
    if ~isempty(old_h);
        h=figure(old_h); 
        cla
    else
        h=figure;
    end
    plot(scale_axis(axis_scale, cash_history), auc_history, 'r', 'LineWidth', 2);
    hold on
    plot(scale_axis(axis_scale, cash_history), auc_history, 'ro', 'MarkerSize', 8, 'MarkerFaceColor', 'r');
    plot(scale_axis(axis_scale, [1 initial_budget]), [1 1]);
    plot(scale_axis(axis_scale, [initial_budget initial_budget]), [0.5 1]);
    plot(scale_axis(axis_scale, [1 initial_budget]), [0.5 0.5]);
    if ~nte
        plot(scale_axis(axis_scale, [1 initial_budget]), [0.5 1], '-.');
    end
    name(name=='_')=' ';
    tt=[upper(name) ': Global score=' num2str(round(10000*final_score)/10000)]; 
    title(tt);
    switch axis_scale
        case 'log'
            xlabel('Log_2(Number of labels queried)');
        case 'sqrt'
            xlabel('Sqrt(Number of labels queried)');
        otherwise
            xlabel('Number of labels queried');
    end
    ylabel('Area under the ROC curve');
    % This is the "rand" algorithm expected results (for the old style
    % learning curves only).
    if ~nte
         cs=1:initial_budget;
         plot(scale_axis(axis_scale, cs), 1-0.5*(1-cs/initial_budget).^2, 'b-');         
    end
return



