function answer_name = send_query(model_dir, data_name, query_dir, answer_dir, query_name, debug)
%answer_name = send_query(model_dir, data_name, query_dir, answer_dir, query_name, debug)
% Emulates sending a query to the server and retrieving the results
    if nargin<6, debug=0; end
    
    % If there is no glop model, just return an empty answer
    if ~exist(data_name, 'file')
        fprintf('[GLOP is not running]\n');
        answer_name='';
        return
    end 
    
    try
        answer_name=query_name;
        % Load the data generating model
        us=findstr(query_name, '_');
        model_name=[model_dir '/' query_name(1:us(length(us))-1) '.mat'];
        if exist(model_name, 'file')
            M=load(model_name); M=M.L;
        else
            M=eval(data_name);
        end
        % Load the query
        Q=query([query_dir '/' query_name]);
        % Process the query
        [Q, M]=process_query(M, Q);
        % Save the model
        save(M, model_name);
        % Save the query
        save(Q, [answer_dir '/' answer_name]); 
        rmzip=~exist('whoisglop', 'file');
        fn=re_zip(answer_dir, query_name, rmzip);     
        fprintf('==> Your answer is in %s\n', fn);
        % Delete plot
        if ~debug
            close(Q.plot_handle);
        end
    catch
        fprintf('[Too old version of Matlab %s to run GLOP]\n', version);
        answer_name='';
        return
    end
return