function CE=cause_effect(a)
% CE=cause_effect(a)
% Finds direct and all causes and effects from the DAG of model a.
% or directly from a DAG G that is a matrix G(i,j) s.t. i->j
% The target is the first variable in G.
% Returns a structure with variables related to the target:
% CE.direct causes
% CE.all_causes
% CE.direct_effects
% CE.all_effects
% CE.spouses
% Numbering is the regular variable numbering, 0 is the target.

% Isabelle Guyon -- isabelle@clopinet.com -- Nov 2007, May 2009, Oct 2009

if ~any(a.target==0)
    fprintf('DBN::cause_effect: WARNING - This network has no specific target numbered 0\n');
    CE=[];
    return;
end
 
G=dag(a);

% Direct effects
de=find(G(1,:));
% Direct causes
dc=find(G(:,1));

% Spouses (parents of the direct effects)
sp=[];
for k=1:length(de)
    sp=[sp; find(G(:,de(k)))];
end
sp=unique(sp);
sp=setdiff(sp, [1]); % remove the target

% All causes
oldparents=dc;
newparents{1}=[];
i=1;
while ~isempty(oldparents) & i<size(G,1)+1
    newparents{i}=[];
    for k=1:length(oldparents)
        op=oldparents(k);
        np=find(G(:,op));
        newparents{i}=[newparents{i}; np];
    end
    oldparents=newparents{i};
    i=i+1;
end
ac=dc;
for k=1:length(newparents)
    ac=[ac; newparents{k}];
end
ac=unique(ac);

% All effects
oldchildren=de;
newchildren{1}=[];
i=1;
while ~isempty(oldchildren) & i<size(G,2)+1
    newchildren{i}=[];
    for k=1:length(oldchildren);
        oc=oldchildren(k);
        nc=find(G(oc,:));
        newchildren{i}=[newchildren{i}, nc];
    end
    oldchildren=newchildren{i};
    i=i+1;
end
ae=de;
for k=1:length(newchildren)
    ae=[ae, newchildren{k}];
end
ae=unique(ae)';
de=de';

% Change the base of the index (0 base)
ae=ae-1;
de=de-1;
ac=ac-1;
dc=dc-1;
sp=sp-1;

CE.direct_causes=dc;
CE.all_causes=ac;
CE.direct_effects=de;
CE.all_effects=ae;
CE.spouses=sp;













