function all_order_ok=check_net(a, reorder)
%all_order_ok = check_net(a, reorder)
% Checks whether the causal order does not violate the structure, i.e. that
% all nodes have their parents earlier in the order
% a -- a query or a bnet object.
% if reorder=1, shows the variable order of the dataset (with 0 for the
% target and the numbering identical to that of the figure at 
% http://www.causality.inf.ethz.ch/data/LUCAS.html

% Isabelle Guyon -- isabelle@clopinet.com -- Nov 2007 - May 2009 -- Sept
% 2009

if any(a.target==0), add1=1; else add1=0; end
if nargin<2, reorder=0; end

bnet=a.bnet;

if iscell(bnet.names)
    names=bnet.names;
else
    names=struct(bnet.names).keys;
end

all_order_ok=1;
for k=1:length(bnet.order)
    i=bnet.order(k);
    % Show the name of the feature and its parents
    ps = bnet.parents{i};
    if reorder
        ii=a.bnet.node_sort(i)-add1;
    else
        ii=i-add1;
    end
    if isempty(ps)
        fprintf('%d:%s\t[Root] ', ii, names{i});
    else
        fprintf('%d:%s\t<== ( ', ii, names{i});
        for j=ps;
            if reorder, jj=a.bnet.node_sort(j)-add1;
            else jj=j-add1; end
            fprintf('%d:%s ', jj, names{j});
        end
        fprintf(') ');
    end
    % Verify the order
    order_ok=all(ismember(ps, bnet.order(1:k-1)));
    all_order_ok=all_order_ok*order_ok;
    if order_ok, fprintf('OK\n'); else fprintf('*** BAD ORDER ***\n'); end
    % Show the properties
    fprintf('\t\t\t-- %s -- %s -- Range=[%g, %g] --\n', var_access(a, ii), var_type(a, ii), var_min(a, ii), var_max(a, ii));
    % Show the CPD for small examples only [fix that and allow SEM]
    if 1==2
        if strcmp(var_type(a, ii), 'binary')
            CPD=get_field(bnet.CPD{i}, 'cpt');
            CPD=CPD(:);
            n=length(CPD)/2;
            CPD=CPD(n+1:2*n);
            if n==1, fprintf('\t\tP(%s=T)=%g\n',names{i}, CPD(1)); end
            if n==2, 
                fprintf('\t\tP(%s=T|%s=F)=%g\n',names{i}, names{ps(1)}, CPD(1)); 
                fprintf('\t\tP(%s=T|%s=T)=%g\n',names{i}, names{ps(1)}, CPD(2)); 
            end
            if n==4 && length(ps)==2, 
                fprintf('\t\tP(%s=T|%s=F, %s=F)=%g\n',names{i}, names{ps(1)}, names{ps(2)}, CPD(1)); 
                fprintf('\t\tP(%s=T|%s=T, %s=F)=%g\n',names{i}, names{ps(1)}, names{ps(2)}, CPD(2)); 
                fprintf('\t\tP(%s=T|%s=F, %s=T)=%g\n',names{i}, names{ps(1)}, names{ps(2)}, CPD(3)); 
                fprintf('\t\tP(%s=T|%s=T, %s=T)=%g\n',names{i}, names{ps(1)}, names{ps(2)}, CPD(4)); 
            end
        end
    end
end
if 1==2, fprintf('\n'); end

if ~all_order_ok
    fprintf('DBN::check_net: ERROR, network structure error\n');
else
    fprintf('DBN::check_net: Network OK\n');
end
            
    