function [q, a]=compute_cost(a, q)
%q=compute_cost(a, q)
%[q, a]=compute_cost(a, q)
% Compute the cost of a query.
% Run this function after the query has been checked to verify all
% manipulations and observations are allowed. Forbidden manipulations or
% observations should be removed from te query.
% If a is provided as an output: updates the internal books of virtual
% cash (a.current_budget).

% Isabelle Guyon -- isabelle@clopinet.com -- May-Oct 2009

q=quantity(q, a); % Let the query know about the number of variables and train/test samples
% Note: at the moment, everything is done is a way that 0 is the (sole)
% target value, if any.
sample_cost=q.samplenum*a.cost_per_sample;
if is_test(q)
    postmanipval(q,[]); % Be careful!!
end
target_cost=q.targetnum*a.cost_per_target_observation;
observ_cost=q.obsernum*a.cost_per_var_observation;
manip_cost=0;
if ~is_test(q)
    manip_cost=q.manipnum*a.cost_per_var_manipulation;
end
total_cost=sample_cost+observ_cost+manip_cost+target_cost;
q=cost(q, total_cost, sample_cost, manip_cost, observ_cost, target_cost); % You must update the query cost
    
new_budget=a.current_budget-total_cost;

if new_budget<0
    %q=premanipval(q, []);
    %q=manipval(q, []);
    %q=postmanipval(q, []);
    q=is_overbudget(q, 1); % Return the results anyways: the problem of Olivier to make verifications
else
    a.current_budget=new_budget;
end
    

return
