function q=manipulate(a, q)
% q=manipulate(a, q)
%Fills in the fields q.manipvar, q.manipval, and q.postmanipvar.
% All the variables except one are manipulated in turn.
% The goal is to predict the values of the unmanipulated variable.

% Isabelle Guyon -- isabelle@clopinet.com -- Oct 2009

tag='manip1';
[manip_var, manip_val, postmanip_var] =reload_sample(a, tag);
if isempty(manip_var)
    sizes=a.bnet.node_sizes; % Maximum variable values
    N=varnum(a);
    facnum=2^(N-1)*N;
    if facnum<=a.test_num
        fprintf('DBN::manipulate: Factorial design\n');
        F=fullfact(sizes);
    else
        fprintf('DBN::manipulate: Random design\n');
        bnum=floor(a.test_num/N)*2;
        F=zeros(bnum, N);
        for k=1:N
            F(:,k)= randi(sizes(k),bnum,1);
        end
    end
    % Manipulate all variables except one and rotate among variables
    M=[];
    T=[];
    for k=1:size(F,2)
        idx_pat=find(F(:,k)==1);
        f=F(idx_pat,:);
        f(:,k)=NaN;
        M=[M; f];
        m=length(idx_pat);
        T=[T; k*ones(m,1)];
    end
    % Replicate to match the size of the observation matrix
    numrep=ceil(a.test_num/size(M,1));
    manip_val=repmat(M, numrep, 1);
    postmanip_var=repmat(T, numrep, 1);
    
    manip_var=a.actionable;
    manip_val=manip_val(1:a.test_num, manip_var);
    postmanip_var=postmanip_var(1:a.test_num);
    
    if ~isempty(manip_val)
        save_sample(a, tag, manip_var, manip_val, postmanip_var);
    end
end

q=manipvar(q, manip_var);
q=manipval(q, manip_val);
q=postmanipvar(q, postmanip_var);
