function xresu = sample_bnet(a, xinit, xmanip)
% SAMPLE_BNET Generate a random sample from discrete valued Bayes net.
% xresu = sample_bnet(a, xinit, xmanip, pm)
% xinit and xmanip are initial values and manipulation values respectively.
% if empty, xresu is just a random sample.
% We assume evidence values positive and 0 or NaN for missing values.
% We convert missing values to [] for compatibility with Murphy's code.
%
% Also accomodates binary +- values, with 0 for no evidence.

% Isabelle Guyon -- Oct 2009 -- isabelle@clopinet.com
% Adapted from sample_bnet of K. Murphy

% Convert
pm=0;
if min(min(xinit))<0 || min(min(xmanip))<0
    fprintf('DBN::sample_bnet: Negative values found, converting to positive values');
    pm=1;
end
xinit(find(xinit==0))=NaN;
xmanip(find(xmanip==0))=NaN;
if pm, 
    xinit=xinit/2+1.5;
    xmanip=xmanip/2+1.5;
end

bnet=a.bnet; 
% Initialize clamped values
idxmanip=find(~isnan(xmanip));
xresu=xinit;
xresu(idxmanip)=xmanip(idxmanip);

% This is not necessary for nets generated with mk_bnet
%node_order = topological_sort(bnet.dag);
node_order=bnet.order(:)'; % This good enough.

for j=node_order
  if isnan(xmanip(j)) % Node is not clampled: update it
    ps = bnet.parents{j};
    if isnan(xinit(j)) || ~isempty(ps) % Node is not initialized yet or is not a root node (initialized root nodes are unchanges)
        e = bnet.equiv_class(j);
        pvals=num2cell(xresu(ps));
        for k=1:length(pvals)
            if isnan(pvals{k})
                pvals{k}=[]; 
            end
        end
        xresu(j) = sample_node(bnet.CPD{e}, pvals);
    end
  end
end

if pm, xresu=(xresu-1.5)*2; end % convert 1 2 to +-1 
