%==========================================================================
% SEM Structural equation model        
%========================================================================== 
% a=SEM(hyper) 
% SEM is derived from "BN". Like a Bayesian network, but the states are
% derived from linear equations.
% To derive a structural equation model from that class
% 1) Create a new directory: @mysem and create in it a file mysem.m 
% containing the 2 lines:
%       classdef mysem < SEM
%       end
% 2) Add in the same directory a file mysem_bnet.m that constructs a bnet 
% or a file mysem_bnet.mat with a saved bnet object.
% See the example below a=sprinkler_bnet(a) for an example and
% the documentation of K. Murphy's Bayesian network toolbox. 
% 3) Add in the same directory a file mysem_W.mat containing a cell array
% with the incoming weights of each node (the bias value is last). If no
% such file is provided, the weights are randomly initialized.

% Uses K. Murphy's Bayesian network toolbox and Jean-Philippe Pellet's code.
% Isabelle Guyon -- isabelle@clopinet.com -- October 2009

classdef SEM < DBN
    
    properties (SetAccess = protected)
        W=[]; % weight matrix.
    end
    
    % <<-------------public methods----------------->> 
    methods
        function v=var_min(a, num)
            if nargin>1
                v=-Inf;
            else
                v=-Inf*ones(1,a.varnum);
            end
        end  
        function v=var_max(a, num)
            if nargin>1
                v=Inf;
            else
                v=Inf*ones(1,a.varnum);
            end
        end
    end
    
    % <<-------------protected methods----------------->> 
    methods (Access = protected)
        
        % Signature of the weight initialization method
        a = init_random_sem_weights(a,nonlinear,sigma);
        
        function a=init(a)
            a=init@DBN(a);
            % Set properties
            a.contact_name='Isabelle Guyon';
            a.contact_url='http://clopinet.com/isabelle/';
            a.title='Structural Equation Model';
            a.authors='Causality Workbench Team';
            a.keywords='structural.equation.models';
            % Set weight values
            a=init_random_sem_weights(a);
        end
        
        function a=load_W(a, filename)
            % Reload a network from a file
            warning off
            load(filename);
            warning on
            a.W=W;
        end
        
        function a=save_W(a, filename)
            % Save a network to a file
            W=a.W;
            save(filename, 'W');
        end
         
        function g=is_gaussian(a)
            g=1;
        end
    end
    
end
