function a = init_random_sem_weights(a,nonlinear,sigma)
%a = init_random_sem_weights(a,nonlinear,sigma)
% sigma is the std dev of the disturbance terms, default 1
% weights{i} is either: - an array [mu, sigma^2] for parentless vars
%                       - an array of beta weigts for each parent plus
%                         sigma^2 for disturbance term

% Author: Jean-Philippe Pellet
% Modified Oct 09: Isabelle Guyon -- isabelle@clopinet.com

weightname=[class(a) '_W.mat'];
if exist([whereis(a) weightname], 'file')==2
    fprintf('SEM::init: Reloading weights %s\n', weightname);
    a=load_W(a, weightname);
    return
end
            
fprintf('BN::init: Initializing weights %s\n', weightname);

dag=a.bnet.dag;

if nargin<3
	sigma=1;
	if nargin<2
		nonlinear=0;
	end
end

if nonlinear
%	funList = {'lin','sin','lin','exp','lin'};
	funList = {'sin'};
end

nvar = size(dag,1);
weights = cell(1,nvar);

for i=1:nvar
	par = parents(dag, i);
	if isempty(par)
		% empty parents, return [mu, sigma^2] as base distribution

		weights{i} = [0 1]; % mean 0, variance 1
		%weights{i} = [(rand(1) - 0.5) * 100 (rand(1) + 0.5) * 1.5];
	else
		%
		npar = length(par);
		% randomly between 0.1 and 0.9 and between -0.1 and 0.9
		%w = 0.1 + rand(1,npar)/1.25;
		w = 0.4 + rand(1,npar)/2;
		%w = w .* sign(rand(1,npar) - 0.5); % change sign with prob 1/2
		if ~nonlinear
			weights{i} = [w sigma]; % if variance == 0, no disturbance term
		else
			funInd=1+round(rand(1,npar)*(length(funList)-1));
			weights{i} = {{funList{funInd}}, [w sigma]}; % if variance == 0, no disturbance term
		end
	end
end

a.W=weights;

save_W(a, [whereis(a) weightname]);

                
