function xresu = sample_bnet(a, xinit, xmanip)
%dat = sample_bnet(a)
% SEM version of sample_bnet, after Jean-Philippe Pellet
% dag is an adacency graph
% weights{i} is either: - an array [mu, sigma^2] for parentless vars
%                       - an array of beta weigts for each parent plus
%                         sigma^2 for disturbance term
% xinit: vector of initialization veriables
% xmanip: vector of manipulation variables
% xresu: one sample.
% Note: missing values are coded as NaN
% For non-linear SEMs, W is a cell array with funs=w{1}; w=w{2};

% Isabelle Guyon -- guyon@clopinet.com -- October 2009

% Fill in missing initial values if necessary
is_initialized=~isnan(xinit);
xinit(~is_initialized)=0;
xresu=update_net(a, xinit, is_initialized);

% Initialize clamped values
is_clamped=~isnan(xmanip);
idxmanip=find(is_clamped);
xresu(idxmanip)=xmanip(idxmanip);

% Update non-clamped values 
xresu=update_net(a, xresu, is_clamped, is_initialized);

return

function xresu=update_net(a, xresu, is_clamped, is_initialized)
bnet=a.bnet; 
weights=a.W;
% This is not necessary for nets generated with mk_bnet
%node_order = topological_sort(a.bnet.dag);
node_order=bnet.order(:)'; % This good enough.
if nargin<3 || isempty(is_clamped), is_clamped=zeros(length(node_order),1); end
if nargin<4, is_initialized=zeros(length(node_order),1); end
for i=node_order
    if ~is_clamped(i) % Node is not clampled: update it
        w = weights{i};
        ps = bnet.parents{i};
        if isempty(ps) && ~is_initialized(i)
            % The node has no parent, draw at random
            if a.is_gaussian
                xresu(i) = w(1) + w(2) * randn(1); % gaussian
            else
                xresu(i) = -1+2*rand; % uniform
            end
        else
            if iscell(w)
                alllin=0;
                funs=w{1};
                w=w{2};
            else
                alllin=1;
            end
            for j=1:length(ps)
                if alllin
                    inc = w(j)*xresu(ps(j));
                else
                    fun=funs{j};
                    if strcmp(fun,'lin')
                        inc = w(j)*xresu(ps(j));
                    elseif strcmp(fun,'exp')
                        inc = w(j)*exp(xresu(ps(j)))/10;
                    elseif strcmp(fun,'log')
                        inc = w(j)*log(abs(xresu(ps(j))) * 4);
                    elseif strcmp(fun,'sin')
                        inc = w(j)*sin(pi*(xresu(ps(j))-.5));
                    else
                        warning(['unknown function id: ' fun]);
                    end
                end
                xresu(i) = xresu(i) + inc;
            end
            xresu(i) = xresu(i) + w(end) * randn(1);
        end
    end % if node not clamped
end


