function [q, a] = process_query(a, q)
%q = process_query(a, q)
%[q, a] = process_query(a, q)
% Inputs:
% a -- A model object.
% q -- A query object or a sample number (if observational data are
% requested only).
% Returns:
% q -- The query object, now containing the answer.
% a -- Optionally get back a with updated budget.

% Isabelle Guyon -- November 2009 -- isabelle@clopinet.com

if is_predict(q)
    % Compute the score
    q=score_result(a, q);
end
if (~is_predict(q) || isempty(q.score))
    fprintf('al_model::process_query: This AL query does not include valid prediction results, we cannot deliver labels.\n');
    q.README=[q.README ' This query does not include valid prediction results, we cannot provide result labels.'];
    return;
elseif is_survey(q)
    % Sample data
    [q, a]=sample(a, q);
end   

% Record the new AUC in the model and compute the global score
a=record_history(a, q);
a=compute_score(a);
% Plot the learning curve
h=plot_learning_curve(a);
% Record the global score and the plot in the query
q=global_score(q, a.current_score, 0); % We do not know how to compute the error bar
q=set_plot_handle(q, h);

return
