function [q, a] = sample(a, q)
%q = sample(a, q)
%[q, a] = sample(a, q)
% Generates data from the model.
% Inputs:
% a -- A model object.
% q -- A query object or a sample number (if observational data are
% requested only).
% Returns:
% q -- The query object, now containing the answer.
% a -- Optionally get back "a" with updated budget.

% Isabelle Guyon -- November 2009 -- isabelle@clopinet.com

if ~is_survey(q) 
    fprintf('al_model::sample: Not a survey query\n');
    return
end
has_target=any(a.target==0);
if ~has_target
    fprintf('al_model::sample: No target found\n');
    return
end

% Important: update the number of variables queried
q=quantity(q, a);
% and verify the range of the variables
q=verify_range(q, a);

% SURVEY DATA ONLY
% ================
% Record as premanipval the values requested
pidx=sanitize_idx(sample(q), a.train_num, a.sample_num);
q=sample(q, pidx);
q=premanipvar(q, 0); 
q=premanipval(q, a.label(pidx,a.targetid)); 
   
% COMPUTE THE COST OF THE QUERY
% =============================
q=samplenum(q, length(setdiff(q.sample, a.queried_samples))); % Do not charge for samples already queried
q=varnum(q, varnum(a)); % Return the variable number for information
[q, a]=compute_cost(a, q);

% KEEP TRACK OF THE SAMPLES QUERIED
% =================================
a.queried_samples=union(a.queried_samples, q.sample);

return

function idx=sanitize_idx(idx, train_num, sample_num)
%idx=sanitize_idx(idx, train_num, sample_num)
% Make sure the indices are correct

% All indices should be integer positive values within bounds
idx0=idx;
idx=unique(round(idx0));
if length(idx0)~=length(idx)
    fprintf('al_model::sample: Duplicate indices eliminated\n');
end
good_idx=find(idx>0 & idx<=sample_num);
idx0=idx;
idx=idx0(good_idx);
if length(idx0)~=length(idx)
    fprintf('al_model::sample: Indices out of bound eliminated\n');
end
% Prevent to query test examples
test_idx=train_num+1:sample_num;
idx0=idx;
idx=setdiff(idx0, test_idx); 

if length(idx0)~=length(idx)
    fprintf('al_model::sample: Test samples queried eliminated\n');
end

if size(idx, 1)<size(idx,2), idx=idx'; end