function [q, a]=score_result(a, q)
%q=score_result(a, q)
% Compute the score for predictions provided by a prediction query.
% There is no budget update because asking to score predictions is "free".
% It can be only done once.

% Isabelle Guyon -- isabelle@clopinet.com -- September 2009

s=[];
e=[];

if isempty(is_predict(q)) || ~is_predict(q)
    error('al_model::score_result: Not a prediction query.');
    return
end

% Retrieve the predicted values
P=postmanipval(q);
% Retrieve the truth values
Q=truth_values(a, q);

lbl_num=length(Q);

% Check the dimensions
if (length(P)>length(Q)), 
    P=P(1:length(Q));
    fprintf('WARNING: al_model::score_result: length of predicted vector larger than truth values: truncated.\n');
    q.README='The length of predicted vector is larger than the number of samples, it was truncated.';
elseif (length(Q)>length(P)), 
    Q=Q(1:length(P)); 
    fprintf('WARNING: al_model::score_result: length of truth value vector larger than predicted vector: aborted.\n');
    q.README='The length of predicted vector is smaller than the number of samples. This query cannot be processed.';
    P=[]; Q=[];
else
    % Compute the score (AUC or BAC)
    idx=1:lbl_num;
    if a.auc_wo_queried_samples
        idx=setdiff(idx, a.queried_samples);
    end
    [s, e] = auc(P(idx), Q(idx));
    %[s, e] = gauc(P(idx), Q(idx));
    %[s, e] = bac(P(idx), Q(idx));

    % Adjust the score (if multiple tasks)
    s=adjust(a, s);
end
    
% Erase the results from the query (not to charge for them)
q=postmanipvar(q, []);
q=postmanipval(q, []);

% Store the score in the query
q=score(q, s, e);

