function [a, manipnum, obsernum, targetnum] = quantity(a, model)
%a=quantity(a)
%a=quantity(a, model)
%[samplenum, manipnum, obsernum, targetnum] = quantity(a)
% "model" is a GLOP model.
%
% Lower level function updating samplenum, manipnum, observnum, and targetnum.
% The calculation is based on actual numbers found in the query,
% and, if "model" given, on the sizes of the default training
% set and test set. These numbers may be changed by models executing the query
% to effective numbers if some manipulations/observations violate constraints.

% Isabelle Guyon -- isabelle@clopinet.com -- Oct 2009

% Note: at present, everything is calculated assuming there is only 1
% target value: 0
if nargin>1
    a=set_sample_num(a, model);
end

% The sample number can also be set by the model for training or test data
if isempty(a.samplenum)
    if is_observation(a)
        a.samplenum=is_observation(a);
    elseif is_experiment(a)
        a.samplenum=size(a.manipval, 1);
    elseif is_survey(a)
        a.samplenum=length(a.sample);
    elseif is_predict(a)
        a.samplenum=0;
    end
end

% Number of manipulations
a.manipnum=length(find(~isnan(a.manipval)));
% Number of observations (if sample number known)
if ~isempty(a.samplenum)
    % Pre-manipulation observational data
    a.obsernum=a.samplenum*size(a.premanipvar,2);
    % Post-manipulation values
    if ~is_test(a)
        a.obsernum=a.obsernum+a.samplenum*size(a.postmanipvar,2);
    end
end

% Number of target observations, if specified
n=0;
if ~isempty(a.samplenum)
    if find(a.premanipvar==0), n=n+1; end
    if ~is_test(a) & find(a.postmanipvar==0), n=n+1; end
    a.targetnum=n*a.samplenum;
end

if nargout>1
    targetnum=a.targetnum;  
	manipnum=a.manipnum;  
    obsernum=a.obsernum;  
    a=a.samplenum;
end

return