function a=save(a, filename, zipme, verbose)
%a=save(a, filename, zipme, verbose)
% Save a query as files with desired extentions

% Isabelle Guyon -- isabelle@clopinet.com -- May 2009

if nargin<3, zipme=0; end
if nargin<4, verbose=0; end

F_exclude={'participant_ID', 'model_name', 'experiment_name', 'date_submitted', 'types', 'plot_handle', 'global_ebar'};

if is_test(a)>0
    a.README=[a.README '== WARNING: TEST' num2str(is_test(a)) '.postmanipval are not given to the participants. These are the values to be predicted.'];
end

if is_overbudget(a)
    a.README=[a.README '== WARNING: Sorry, your budget would be exceeded if we processed this query. Canceled.'];
else
    F_exclude=union(F_exclude, 'is_overbudget');
end

if is_predict(a)==0, 
    F_exclude=union(F_exclude, {'score', 'ebar'}); 
end

if strcmp(a.type, 'AL')
    F_exclude=union(F_exclude, {'premanipvar', 'postmanipvar', 'type', 'targetnum', 'targetcost', 'samplenum', 'samplecost', 'obsernum', 'obsercost', 'manipnum', 'manipcost', 'varnum'}); 
end

F=setdiff(fields(a), F_exclude);

if isempty(a.README)
    a.README='Query processed successfully';
end

for k=1:length(F)
    if ischar(a.(F{k}))
        fp=fopen([filename '.' F{k}], 'w');
        fprintf(fp, '%s\n', a.(F{k}));
        fclose(fp);
    elseif ~isempty(a.(F{k}))
        if strcmp(a.type, 'AL') && strcmp(F{k}, 'premanipval')
            matrix_data_write([filename '.label'], a.(F{k}), [], verbose);
        elseif strcmp(a.type, 'AL') && strcmp(F{k}, 'postmanipval')
            matrix_data_write([filename '.predict'], a.(F{k}), [], verbose);
        else
            matrix_data_write([filename '.' F{k}], a.(F{k}), [], verbose);
        end
    end
end

if zipme
    [dirname, filename]=split_filename(filename);
    rmfiles=1;
    re_zip(dirname, filename, rmfiles);
end