function q=set_sample_num(q, model)
%q=set_sample_num(q, model)
%
% Set the sample numbers of TRAIN (ntr) and TEST (nte) if is_train or
% is_test or set it properly otherwise. Nvar is the number of variables
% (except the target). Ntarg is the number of targets.
% Perform a few sanity checks.

% Isabelle Guyon -- isabelle@clopinet.com - Oct 2009

Nvar = model.varnum;
Ntarg =  model.targnum;
ntr = model.train_num;
nte = model.test_num;

% Number of samples requested
if isnumeric(q)    
    p=q;
    q=query;
    q=samplenum(q, p); % Ask for a certain number of samples
elseif is_test(q)
    p=nte;
    q=samplenum(q, p); % No manipulation yet => querynum=samplenum
elseif is_train(q)
    p=ntr;
    q=samplenum(q, p); % Querynum=samplenum (no manip)
end
if isempty(samplenum(q)) % Double check
    error('query::set_sample_num: Empty number of samples');
end

% Default variables requested, premanip
if is_train(q)==1 || is_test(q) || is_observation(q)
    if isempty(premanipvar(q))
        q=premanipvar(q, 1:Nvar);
        warning('No premanipvar found, using 1:Nvar by default');
    end
elseif is_train(q)==2
    % For the training set, TRAIN2 means labeled data // this happens not
    % for data with no target value, this is an internal track
    if Ntarg>1, warning('More than one target value is not supported'); Ntarg=1; end
    q=premanipvar(q, (1-Ntarg):Nvar);
    warning('No premanipvar found, using 0:Nvar by default (labeled training data)');
end

% We authorize to buy observations of the premanipulation target values 
% of TRAIN, OBS and TESTn, n>1. For TEST1 (unmanipulated data), 
% those must be predicted, hence cannot be observed
if is_test(q)==1
    idx=premanipvar(q);
    tidx=find(idx==0); % Does not happen for data without target
    n=length(idx);
    if ~isempty(tidx)
        warning('Cannot deliver the target value for TEST1, unmanipulated test data');
        idx=idx([1:tidx-1,tidx+1:n]);
    end
    q=premanipvar(q, idx);
end

% Check that the variable range is correct
varlist={'manipvar', 'premanipvar', 'postmanipvar'};
for k=1:length(varlist)
    idx=feval(varlist{k}, q);
    if ~isempty(setdiff(idx, 0:Nvar))
        warning(['Wrong variable index in ' varlist{k} ' variables']);
    end
end

function warning(message)
fprintf('query::set_sample_num: %s\n', message);