function a=verify_range(a, vmin, vmax, has_target)
%a=verify_range(a, vmin, vmax, has_target)
%a=verify_range(a, model) % where "model" is a GLOP model
% Verify the variable range.
% vmin -- variable minimum value(s)
% vmax -- variable minimum value(s)
% has_target -- 1 if the target index 0 is used
% Rescale variables to fit in the range, if needed.

% Isabelle Guyon - isabelle@clopinet.com - October 2009

if ~isempty(strfind(a.type, 'PREDICT')), return, end

if isobject(vmin)
    model=vmin;
    vmin=model.var_min;
    vmax=model.var_max;
    has_target=any(model.target==0);
else
    if nargin<3, vmin=-Inf; vmax=Inf;  end
    if nargin<4, has_target=0; end
end

varval={'premanip', 'manip', 'postmanip'};
for kk=1:length(varval)
    [sidx]=sanitize_idx(a.([varval{kk} 'var']), has_target);
    true_max=length(vmin);
    maxs=max(sidx);
    if maxs>true_max
        vmin(true_max+1:maxs)=-Inf;
        vmax(true_max+1:maxs)=Inf;
        fprintf('query::verify_range: index out of bounds ingnored\n');
    end
    X=a.([varval{kk} 'val']);
    if ~isempty(X)
        vmini=vmin(sidx);
        vmaxi=vmax(sidx);
        for k=1:size(X,2)
            x=X(:,k);
            if any(x>vmaxi(k)) || any(x<vmini(k))
                % Rescale
                x0=min(x);
                x1=max(x);
                x=(((x-x0)/(x1-x0))*(vmaxi(k)-vmini(k)))+vmini(k);
                X(:,k)=x;
            end
        end
        a.([varval{kk} 'val'])=X;
    end
end

function idx=sanitize_idx(idx, has_target)
    if isempty(idx), return, end
    if has_target, idx=idx+1; 
    elseif any(idx==0)
        idx=idx+1;
    end
    
    