function [area, sigma] = gauc(Output, Target)
%[area, sigma] = gauc(Output, Target)
% This is the algorithm proposed for 
% computing the generalized AUC and the error bar.
% It is assumed that the outputs provide a score
% with the examples of the good (positive) class having the highest score.

% Isabelle Guyon -- isabelle@clopinet.com -- October 2009

area=[];
sigma=[]; % No error bars yet...

% This is hard to vectorize, we just loop if multiple columns for outputs
[nn,pp]=size(Output);
p=1;
if nn~=1 & pp~=1
    p=pp;
elseif nn==1
    Output=Output';
    Target=Target';
end

% Compute the normalization factor
Smax=compute_rank(Target); 
DPmax=sum(Smax.*Target);
Smin=compute_rank(-Target); 
DPmin=sum(Smin.*Target);
    
for kk=1:p
    
    output=Output(:,kk);
    S=compute_rank(output);   

    % Compute the dot product:
    DP=sum(S.*Target); 
    area(kk)=(DP-DPmin)/(DPmax-DPmin);
 
end % for kk



function S=compute_rank(output)
[u,i]=sort(output); % sort outputs, best comes last (u=sorted vals, i=index)
n=length(output);

uval=unique(output);
S=zeros(size(output));
% Test whether there are ties 
if length(uval)==n
	S(i)=1:n;   % compute the ranks of the outputs (no ties)
else

    % Average the ranks for the ties 
    oldval=u(1);
    newval=u(1);
    R=1:n;
    k0=1;
    for k=2:n
        newval=u(k);
        if newval==oldval
            % moving average
            R(k0:k)=R(k-1)*(k-k0)/(k-k0+1)+R(k)/(k-k0+1);
        else
            k0=k;
        end
        oldval=newval;
    end
    S(i)=R;
end


