% -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-
%                               SAMPLE CODE FOR THE "
%                           "ACTIVE LEARNING CHALLENGE"
% -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-

% DISCLAIMER: ALL INFORMATION, SOFTWARE, DOCUMENTATION, AND DATA ARE PROVIDED "AS-IS" 
% ISABELLE GUYON AND/OR OTHER CONTRIBUTORS DISCLAIM ANY EXPRESSED OR IMPLIED WARRANTIES, 
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
% FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY OF NON-INFRIGEMENT OF ANY
% THIRD PARTY'S F
% INTELLECTUAL PROPERTY RIGHTS. IN NO EVENT SHALL ISABELLE GUYON AND/OR OTHER CONTRIBUTORS 
% BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF SOFTWARE, DOCUMENTS, 
% MATERIALS, PUBLICATIONS, OR INFORMATION MADE AVAILABLE FOR THE CHALLENGE. 

% Author: Isabelle Guyon -- November 2009 -- causality@clopinet.com

close all;
fclose all;
%clear all;
%clear classes;
%clc

%% -o-|-o-|-o-|-o-|-o-|-o-|-o- BEGIN USER-PREFERENCES -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-

% 1) Directory tree (you do not need to change that)
% -----------------
%       The present set up supposes the sample_code.m file is in a 
% directory <my_root> from which you are running at the Matlab prompt:
%
% > sample_code;

my_root     = '.';  % Change that to the directory of your project

this_dir=pwd; cd(my_root); my_root=pwd; cd(this_dir); % Get the absolute path

data_dir    = [my_root '/Data'];      % Directory of the data files
label_dir   = [my_root '/Labels'];    % Directory of the label files [you should not ned this unless you have an old Matlab version]
func_dir    = [my_root '/Functions']; % Directory of the sample code functions
query_dir   = [my_root '/Queries'];   % Directory of the queries
answer_dir  = [my_root '/Answers'];   % Directory of the query answers 
plot_dir    = [my_root '/Plots'];     % Directory of the learning curves 
glop_dir    = [my_root '/GLOP'];      % Generative Learning Objects Package (GLOP).
clop_dir    = [my_root '/CLOP'];      % Challenge Learning Objects Package (CLOP).
model_dir   = [my_root '/Models'];    % Where GLOP models are stored.

% 2) Information about the datasets (you should not need to change that)
% ---------------------------------
% List of datasets
dataset     = {'alex', 'hiva', 'ibn_sina', 'nova', 'orange', 'sylva', 'zebra'};
% Indices of one example of the positive class for each dataset (seeds)
seeds       = [1     1     4    11    54     4     23];
% Number of training examples:
train_num   = [5000 21339 10361 9733 25000 72626 30744];
% Scaling of the X axes of the learning curves 
xaxes       = {'log', 'log', 'log', 'log', 'log', 'log', 'log'};

% 3) Setup of your experiment
% ---------------------------
%%% Choose the subset of the datasets that you want to work on %%%
% Use e.g.: data_list=1:length(dataset); if you want all of them
% or data_list=[1 3 4]; if you want only alex, ibn_sina and nove.
data_list       =   1; %1:length(dataset);
% Your nickname you used on the active learning challenge website
% (it does not matter if you choose something else here)
my_workbench_id = 'myname';         
% The name of the machine learning algorithm (if you are running
% CLOP, whoisclop will give you a list of ml_algo). The 'zarbi' example
% provided with the sample code is coupled with a simple "uncertainty
% sampling" strategy. You can try also 'rand' (random predictions), 
% 'zarbip' (passive machine learning asking for all the labels at once)
% or 'zarbir' (random sampling strategy).
ml_algo         = 'zarbi'; 
% The name of your experiment (by default: the same as ml_algo but you
% can change that.
experiment_name = ml_algo;         
% The date (if you want a more precise date and time, change that)
the_date        = datestr(now, 'yyyy-mm-dd');  

% 4) Use GLOP (optional)
% -----------
% GLOP is a package, which includes data generating models used in the
% challenge. You can run the sample code without it by manually submitting
% queries generated by the code to the web server and putting the resulting 
% answer zip files in the "Answers" directory. 
% To use GLOP with your development models, change this flag to 1:
glop_on=0;
% Older experiments with the same date are overwritten if restart_from_scratch=1
% Use 0 to restart where an experiment was interrupted to avoid recalculating everything.
if glop_on
    restart_from_scratch=1; % You may change that
else
    restart_from_scratch=0;
end
% Note: Not restarting from scratch may result in undesirable side effects
% because the GLOP model internal state may be incorrect and result in
% a wrong cost accounting. Use with care if glop_on==1.

% 4) Use CLOP (optional)
% -----------
% CLOP is a package of machine learning algorithms. You do not
% need CLOP to run this sample code, we provide one toy machine learning
% object @zarbi as an example. But many other algorithms are available in
% CLOP, downloadable fr                                                                                                                 om http://clopinet.com/CLOP/.
% Place the CLOP package in the CLOP subdirectory.

%% -o-|-o-|-o-|-o-|-o-|-o-|-o- END USER-PREFERENCES -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-

%% Initializations
warning off; addpath(my_root); addpath(func_dir); addpath(glop_dir); addpath(clop_dir); addpath(label_dir); warning on;
if ~exist('whoisclop', 'file') && exist('use_spider_clop.m', 'file'), use_spider_clop(clop_dir); end
if glop_on && exist('use_glop.m', 'file')
    if ~exist('whoisglop', 'file'), use_glop(glop_dir); end
else
    use_glop(glop_dir, 1); 
end
if ~exist(data_dir, 'dir') == 7, fprintf('[No data directory!]\n');  return ; end
makedir(query_dir);
makedir(answer_dir);
makedir(model_dir);
makedir(plot_dir);
makedir(label_dir);
debug=0;
warning off

%% Advanced users only, define your own CLOP model:
global mymodel;
%mymodel=chain({standardize, s2n('f_max=10'), kridge('degree=2')});

%% LOOP OVER DATASETS 
% ===================
warning on
for k=data_list
    
    data_name   = dataset{k};
    query_name  = [my_workbench_id '_' data_name '_' experiment_name '_' the_date];
    
    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n');
    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-      %s      -o-|-o-|-o-|-o-|-o-|-o-\n', upper(data_name));
    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n\n');
    
    % Cleanup old files
    if restart_from_scratch
        cleanup(model_dir, query_dir, answer_dir, [my_workbench_id '_' data_name '_' experiment_name]);
    end
    
    % Load the data
    X=load_data(data_dir, data_name);
    if isempty(X), 
        fprintf('==> Empty dataset %s\n', data_name);
        return; 
    end
    [P, N]=size(X);
    if isempty(train_num), ntr=P; else ntr=train_num(k); end
    nte=P-ntr;
    
    % Choose the number of labels queried at each iteration
    [label_num, chunk_num]=choose_data_split(ml_algo, xaxes(k), ntr);

    % Create an initial training set
    known_idx=seeds(k);
    Ytr=1; % The seed examples is of class 1
    
    % Load the true labels (only for debug purpose)
    if debug
        M=eval(data_name);
        Y=M.label;
    end
    
    score=[];
    global_score=[];
    cash_spent=[];
    cash_spent(1)=1; % we pay for the seed
    old_h=[];        % plot handle
    
    % Perform active learning iterations
     for i=1:chunk_num+1  
        fprintf('\n-o-|-o-|-o-     %s      -o-|-o-|-o-\n', query_name);
        if i==chunk_num+1
            fprintf('\n-o-|-o-|-o-     Final predictions      -o-|-o-|-o-\n\n');
        else
            fprintf('\n-o-|-o-|-o-     Active learning, iteration %d/%d      -o-|-o-|-o-\n\n', i, chunk_num);
        end
        query_name_i=[query_name '-' sprintf('%02d', i)];
        answer_name=query_name_i;
        
        % Bypass creating and sending the query if we have already the answer
        if ~probe_query(answer_dir, query_name_i) || restart_from_scratch
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%%%          Make predictions & rank samples                %%%%%            
            %
            % Here you can replace this by your own code:
            %
            [Predicted_values, Ranked_samples]=active_learning(ml_algo, X, known_idx, Ytr, nte);
            %
            % You must return:
            %   - Predicted_values: a vector of predictions of the label
            %   for all the lines in the data matrix
            %   - Ranked_samples: an ordered list of samples you wish to
            %   query next (the most desired come first).
            %
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            if isempty(Predicted_values), return; end
            
            % Overwrite the predicted values (for debug purpose)
            if debug && strcmp(ml_algo, 'opt')
                    Predicted_values=Y;
            end

            % Create a query (queries should always include predictions on
            % all the samples in addition to the list of labels requested)
            if i>chunk_num 
                Samples_queried=[];
            else
                if label_num(i)>length(Ranked_samples)
                    fprintf('==> Over budget!\n');
                    Samples_queried=[];
                else
                    Samples_queried=Ranked_samples(1:label_num(i));
                end
            end
            fprintf('==> Number of labels queried: %d\n', length(Samples_queried));
            
            % Create a zip file with the query in the Queries directory
            make_query(query_dir, query_name_i, Predicted_values, Samples_queried);
            
            %if i==4, error('asd'); end
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%%%%% Here the query must be sent and the result retrieved %%%%%%
            % If you have GLOP, the query is executed locally.            
            answer_name = send_query(model_dir, data_name, query_dir, answer_dir, query_name_i, debug);
            % If you don't have GLOP, you must submit the query to the server.            
            % %%%%%     Place the results in the Answers directory       %%%%%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        end
               
        % Get the result of the query from the Answers directory
        [Y_received, Samples_received, score(i), global_score(i), cost, stop]=get_answer(answer_dir, answer_name, ...
            data_name, Predicted_values, Samples_queried, cash_spent, score, P-nte, xaxes{k}, known_idx); % the rest of the arguments are for the back-door
        
        
        
        if stop, return; end
        
        fprintf('==> AUC score = %g\n', score(i));
        fprintf('==> Global score (ALC) = %g\n', global_score(i));
        fprintf('==> Number of labels received: %d\n', length(Samples_received));
        if isempty(Y_received), 
            fprintf('==> No labels received!\n');
        end
        cash_spent(i+1)=cash_spent(i)+cost;
        fprintf('==> Total cash spent: %d\n', cash_spent(i+1));
        % Note: the samples received may not coincide with those queried,
        % if you ask for "forbidden" samples (like test examples), or if
        % you ask for labels you asked for previously.
        % The cash spent will be the number of samples received minus
        % those asled previously that you paid for already.  

        % Create a new training set
        known_idx=[known_idx; Samples_received];
        Ytr=[Ytr; Y_received]; 

        % Verify we got the right stuff (for debug purpose)
        if debug
            if ~all(Ytr==Y(known_idx))
                fprintf('==> AAAAARGH!!\n')
            end
        end
        
        % Draw the learning curve
        h=plot_learning_curve(query_name, cash_spent, score, P-nte, global_score(length(global_score)), xaxes{k}, nte, old_h);
        saveas(h, [plot_dir '/' query_name '.png']);
        old_h=h;
     end    % for loop over chunks 
end         % for loop over datasets

%%%%%%%%%%%%%%%%%%%%% END MAIN %%%%%%%%%%%%%%%%%%%%%%%
